@@ -102,6 +102,7 @@ static void image_loader_size_cb(GdkPixbufLoader *loader,
 			}
 		
 		gdk_pixbuf_loader_set_size(loader, nw, nh);
+		il->shrunk = TRUE;
 		}
 }
 
@@ -220,47 +221,66 @@ static gint image_loader_begin(ImageLoader *il)
 		return FALSE;
 		}
 
-	if (!gdk_pixbuf_loader_write(il->loader, buf, b, NULL))
+	if (gdk_pixbuf_loader_write(il->loader, buf, b, NULL))
 		{
-		image_loader_stop(il);
-		return FALSE;
-		}
-
-	il->bytes_read += b;
-
-	/* read until size is known */
-	while (il->loader && !gdk_pixbuf_loader_get_pixbuf(il->loader) && b > 0)
-		{
-		b = read(il->load_fd, &buf, sizeof(buf));
-		if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b, NULL)))
-			{
-			image_loader_stop(il);
-			return FALSE;
-			}
 		il->bytes_read += b;
+
+		if (b < sizeof(buf))
+			{
+			/* end of file already */
+
+			image_loader_stop(il);
+
+			if (!il->pixbuf) return FALSE;
+
+			image_loader_done_delay(il);
+			return TRUE;
+			}
+		else
+			{
+			/* larger file */
+
+			/* read until size is known */
+			while(il->loader && !gdk_pixbuf_loader_get_pixbuf(il->loader) && b > 0)
+				{
+				b = read(il->load_fd, &buf, sizeof(buf));
+				if (b < 0 || (b > 0 && !gdk_pixbuf_loader_write(il->loader, buf, b, NULL)))
+					{
+					image_loader_stop(il);
+					return FALSE;
+					}
+				il->bytes_read += b;
+				}
+			if (!il->pixbuf) image_loader_sync_pixbuf(il);
+
+			if (il->bytes_read == il->bytes_total || b < sizeof(buf))
+				{
+				/* done, handle (broken) loaders that do not have pixbuf till close */
+				image_loader_stop(il);
+
+				if (!il->pixbuf) return FALSE;
+
+				image_loader_done_delay(il);
+				return TRUE;
+				}
+
+			if (!il->pixbuf)
+				{
+				image_loader_stop(il);
+				return FALSE;
+				}
+
+			/* finally, progressive loading :) */
+			il->idle_id = g_idle_add_full(il->idle_priority, image_loader_idle_cb, il, NULL);
+			return TRUE;
+			}
 		}
-	if (!il->pixbuf) image_loader_sync_pixbuf(il);
-
-	if (il->bytes_read == il->bytes_total || b < 1)
-		{
-		/* done, handle (broken) loaders that do not have pixbuf till close */
-		image_loader_stop(il);
-
-		if (!il->pixbuf) return FALSE;
-
-		image_loader_done_delay(il);
-		return TRUE;
-		}
-
-	if (!il->pixbuf)
+	else
 		{
 		image_loader_stop(il);
 		return FALSE;
 		}
 
-	/* finally, progressive loading :) */
-	il->idle_id = g_idle_add_full(il->idle_priority, image_loader_idle_cb, il, NULL);
-
 	return TRUE;
 }
 
@@ -287,6 +307,8 @@ static gint image_loader_setup(ImageLoader *il)
 	g_signal_connect(G_OBJECT(il->loader), "size_prepared",
 			 G_CALLBACK(image_loader_size_cb), il);
 
+	il->shrunk = FALSE;
+
 	return image_loader_begin(il);
 }
 
@@ -314,6 +336,7 @@ ImageLoader *image_loader_new(const gchar *path)
 
 	il->requested_width = 0;
 	il->requested_height = 0;
+	il->shrunk = FALSE;
 
 	return il;
 }
