@@ -30,8 +30,8 @@
 #define IMAGE_SIZE_W 200
 #define IMAGE_SIZE_H 200
 
-#define DEF_PROPERTY_WIDTH  600
-#define DEF_PROPERTY_HEIGHT 400
+#define DEF_PROPERTY_WIDTH  510
+#define DEF_PROPERTY_HEIGHT 390
 
 typedef struct _TabData TabData;
 struct _TabData
@@ -232,7 +232,7 @@ static void info_tab_general_image(InfoData *id, gpointer data)
 
 	if (!tab->compression_done && mem_size > 0)
 		{
-		buf = g_strdup_printf("%.1f%%", (float)tab->byte_size / mem_size * 100.0);
+		buf = g_strdup_printf("%.1f%%", (double)tab->byte_size / mem_size * 100.0);
 		gtk_label_set_text(GTK_LABEL(tab->label_compression), buf);
 		g_free(buf);
 
@@ -501,7 +501,7 @@ static void info_window_dnd_data_set(GtkWidget *widget, GdkDragContext *context,
 		g_list_free(list);
 
 		gtk_selection_data_set(selection_data, selection_data->target,
-				       8, (guchar *)text, len);
+				       8, text, len);
 		g_free(text);
 		}
 }
