@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2006 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -28,6 +28,8 @@
 #include "ui_tabcomp.h"
 #include "ui_utildlg.h"
 
+#include "icons/config.xpm"
+
 #include <math.h>
 
 
@@ -85,7 +87,10 @@ static gint thumb_max_width_c;
 static gint thumb_max_height_c;
 static gint enable_thumb_caching_c;
 static gint enable_thumb_dirs_c;
+static gint thumbnail_fast_c;
+#if 0
 static gint use_xvpics_thumbnails_c;
+#endif
 static gint thumbnail_spec_standard_c;
 static gint enable_metadata_dirs_c;
 static gint show_dot_files_c;
@@ -235,7 +240,10 @@ static void config_window_apply(void)
 	thumb_max_height = thumb_max_height_c;
 	enable_thumb_caching = enable_thumb_caching_c;
 	enable_thumb_dirs = enable_thumb_dirs_c;
+	thumbnail_fast = thumbnail_fast_c;
+#if 0
 	use_xvpics_thumbnails = use_xvpics_thumbnails_c;
+#endif
 	thumbnail_spec_standard = thumbnail_spec_standard_c;
 	enable_metadata_dirs = enable_metadata_dirs_c;
 	show_dot_files = show_dot_files_c;
@@ -394,7 +402,7 @@ static void add_quality_menu(GtkWidget *table, gint column, gint row, const gcha
 			 G_CALLBACK(quality_menu_cb), option_c);
 
 	gtk_table_attach(GTK_TABLE(table), combo, column + 1, column + 2, row, row + 1,
-			 GTK_EXPAND | GTK_FILL, 0, 0, 0);
+			 GTK_EXPAND | GTK_FILL, FALSE, 0, 0);
 	gtk_widget_show(combo);
 }
 
@@ -505,7 +513,7 @@ static void add_thumb_size_menu(GtkWidget *table, gint column, gint row, gchar *
 			 G_CALLBACK(thumb_size_menu_cb), NULL);
 
 	gtk_table_attach(GTK_TABLE(table), combo, column + 1, column + 2, row, row + 1,
-			 GTK_EXPAND | GTK_FILL, 0, 0, 0);
+			 GTK_EXPAND | GTK_FILL, FALSE, 0, 0);
 	gtk_widget_show(combo);
 }
 
@@ -769,7 +777,7 @@ static void config_window_create(void)
 	gtk_window_set_wmclass(GTK_WINDOW(configwindow), "config", "GQview");
 	gtk_container_set_border_width(GTK_CONTAINER(configwindow), PREF_PAD_BORDER);
 
-	window_set_icon(configwindow, PIXBUF_INLINE_ICON_CONFIG, NULL);
+	window_set_icon(configwindow, (const gchar **)config_xpm, NULL);
 
 	win_vbox = gtk_vbox_new(FALSE, PREF_PAD_SPACE);
 	gtk_container_add(GTK_CONTAINER(configwindow), win_vbox);
@@ -853,11 +861,16 @@ static void config_window_create(void)
 	subgroup = pref_box_new(subgroup, FALSE, GTK_ORIENTATION_VERTICAL, PREF_PAD_GAP);
 	pref_checkbox_link_sensitivity_swap(button, subgroup);
 
-	button = pref_checkbox_new_int(subgroup, _("Cache thumbnails into .thumbnails"),
-				       enable_thumb_dirs, &enable_thumb_dirs_c);
+	pref_checkbox_new_int(subgroup, _("Cache thumbnails into .thumbnails"),
+			      enable_thumb_dirs, &enable_thumb_dirs_c);
 
-	button = pref_checkbox_new_int(subgroup, _("Use xvpics thumbnails when found (read only)"),
-				       use_xvpics_thumbnails, &use_xvpics_thumbnails_c);
+#if 0
+	pref_checkbox_new_int(subgroup, _("Use xvpics thumbnails when found (read only)"),
+			      use_xvpics_thumbnails, &use_xvpics_thumbnails_c);
+#endif
+
+	pref_checkbox_new_int(group, _("Faster jpeg thumbnailing (may reduce quality)"),
+			      thumbnail_fast, &thumbnail_fast_c);
 
 	group = pref_group_new(vbox, FALSE, _("Slide show"), GTK_ORIENTATION_VERTICAL);
 
@@ -895,7 +908,7 @@ static void config_window_create(void)
 
 	zoom_increment_c = zoom_increment;
 	spin = pref_spin_new(group, _("Zoom increment:"), NULL,
-			     0.1, 4.0, 1.0, 1, (float)zoom_increment / 10.0,
+			     0.1, 4.0, 1.0, 1, (double)zoom_increment / 10.0,
 			     G_CALLBACK(zoom_increment_cb), NULL);
 	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
 
@@ -1073,7 +1086,7 @@ static void config_window_create(void)
 	label = pref_table_label(table, 1, 0, _("Menu name"), 0.0);
 	pref_label_bold(label, TRUE, FALSE);
 
-	label = pref_table_label(table, 2, 0, _("Command Line"), 0.0);
+	label = pref_table_label(table, 2, 0, ("Command Line"), 0.0);
 	pref_label_bold(label, TRUE, FALSE);
 
 	for (i = 0; i < GQVIEW_EDITOR_SLOTS; i++)
@@ -1089,7 +1102,7 @@ static void config_window_create(void)
 		gtk_widget_set_size_request(editor_name_entry[i],80,-1);
 		if (editor_name[i]) gtk_entry_set_text(GTK_ENTRY(editor_name_entry[i]),editor_name[i]);
 		gtk_table_attach(GTK_TABLE (table),editor_name_entry[i],1,2,i+1,i+2,
-				 GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				 GTK_FILL | GTK_EXPAND, FALSE, 0, 0);
 		gtk_widget_show(editor_name_entry[i]);
 
 		editor_command_entry[i] = gtk_entry_new();
@@ -1098,7 +1111,7 @@ static void config_window_create(void)
 		tab_completion_add_to_entry(editor_command_entry[i], NULL, NULL);
 		if (editor_command[i]) gtk_entry_set_text(GTK_ENTRY(editor_command_entry[i]), editor_command[i]);
 		gtk_table_attach(GTK_TABLE (table),editor_command_entry[i],2,3,i+1,i+2,
-				 GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				 GTK_FILL | GTK_EXPAND, FALSE, 0, 0);
 		gtk_widget_show(editor_command_entry[i]);
 		}
 
@@ -1301,7 +1314,7 @@ void show_about_window(void)
 
 	buf = g_strdup_printf(_("GQview %s\n\nCopyright (c) %s John Ellis\nwebsite: %s\nemail: %s\n\nReleased under the GNU General Public License"),
 			      VERSION,
-			      "2006",
+			      "2005",
 			      "gqview.sourceforge.net",
 			      "gqview@users.sourceforge.net");
 	label = gtk_label_new(buf);
