@@ -194,9 +194,6 @@ void save_options(void)
 	fprintf(f,"#                         GQview config file         version %7s #\n", VERSION);
 	fprintf(f,"######################################################################\n");
 	fprintf(f,"\n");
-	fprintf(f,"# Note: This file is autogenerated. Options can be changed here,\n");
-	fprintf(f,"#       but user comments and formatting will be lost.\n");
-	fprintf(f,"\n");
 	fprintf(f,"##### General Options #####\n\n");
 
 	write_int_option(f, "layout_style", layout_style);
@@ -241,6 +238,7 @@ void save_options(void)
 	write_int_option(f, "thumbnail_height", thumb_max_height);
 	write_bool_option(f, "cache_thumbnails", enable_thumb_caching);
 	write_bool_option(f, "cache_thumbnails_into_dirs", enable_thumb_dirs);
+	write_bool_option(f, "thumbnail_fast", thumbnail_fast);
 	write_bool_option(f, "use_xvpics_thumbnails", use_xvpics_thumbnails);
 	write_bool_option(f, "thumbnail_spec_standard", thumbnail_spec_standard);
 	fprintf(f,"\n");
@@ -305,7 +303,7 @@ void save_options(void)
 	filter_write_list(f);
 
 	fprintf(f,"\n##### External Programs #####\n");
-	fprintf(f,"# Maximum of 10 programs (external_1 through external_10)\n");
+	fprintf(f,"# Maximum of 8 programs (external_1 through external 8)\n");
 	fprintf(f,"# format: external_n: \"menu name\" \"command line\"\n\n");
 
 	for (i = 0; i < GQVIEW_EDITOR_SLOTS; i++)
@@ -453,6 +451,8 @@ void load_options(void)
 			"cache_thumbnails", value, enable_thumb_caching);
 		enable_thumb_dirs = read_bool_option(f, option,
 			"cache_thumbnails_into_dirs", value, enable_thumb_dirs);
+		thumbnail_fast = read_bool_option(f, option,
+			"thumbnail_fast", value, thumbnail_fast);
 		use_xvpics_thumbnails = read_bool_option(f, option,
 			"use_xvpics_thumbnails", value, use_xvpics_thumbnails);
 		thumbnail_spec_standard = read_bool_option(f, option,
