@@ -153,7 +153,7 @@ static gchar *thumb_std_cache_path(const gchar *path, const gchar *uri, gint loc
 		cache_base = g_strconcat(homedir(), "/", THUMB_FOLDER, "/", cache_subfolder, NULL);
 		}
 
-	md5_get_digest((guchar *)uri, strlen(uri), digest);
+	md5_get_digest(uri, strlen(uri), digest);
 	md5_text = md5_digest_to_text(digest);
 
 	if (cache_base && md5_text)
@@ -442,7 +442,7 @@ static gint thumb_loader_std_scale_aspect(gint req_w, gint req_h, gint old_w, gi
 	return (*new_w != old_w || *new_h != old_h);
 }
 
-static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf)
+static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf, gint shrunk)
 {
 	GdkPixbuf *pixbuf_thumb = NULL;
 	GdkPixbuf *result;
@@ -451,55 +451,49 @@ static GdkPixbuf *thumb_loader_std_finish(ThumbLoaderStd *tl, GdkPixbuf *pixbuf)
 	sw = gdk_pixbuf_get_width(pixbuf);
 	sh = gdk_pixbuf_get_height(pixbuf);
 
-	if (tl->cache_enable)
+	if (tl->cache_enable && !tl->cache_hit &&
+	    (sw >= THUMB_SIZE_NORMAL || sh >= THUMB_SIZE_NORMAL || shrunk))
 		{
-		if (!tl->cache_hit)
+		gint cache_w, cache_h;
+		gint thumb_w, thumb_h;
+
+		if (tl->requested_width > THUMB_SIZE_NORMAL || tl->requested_height > THUMB_SIZE_NORMAL)
 			{
-			gint cache_w, cache_h;
-
-			if (tl->requested_width > THUMB_SIZE_NORMAL || tl->requested_height > THUMB_SIZE_NORMAL)
-				{
-				cache_w = cache_h = THUMB_SIZE_LARGE;
-				}
-			else
-				{
-				cache_w = cache_h = THUMB_SIZE_NORMAL;
-				}
-
-			if (sw > cache_w || sh > cache_h)
-				{
-				gint thumb_w, thumb_h;
-
-				if (thumb_loader_std_scale_aspect(cache_w, cache_h, sw, sh,
-								  &thumb_w, &thumb_h))
-					{
-					pixbuf_thumb = gdk_pixbuf_scale_simple(pixbuf, thumb_w, thumb_h,
-									       (GdkInterpType)thumbnail_quality);
-					}
-				else
-					{
-					pixbuf_thumb = pixbuf;
-					g_object_ref(G_OBJECT(pixbuf_thumb));
-					}
-
-				thumb_loader_std_save(tl, pixbuf_thumb);
-				}
+			cache_w = cache_h = THUMB_SIZE_LARGE;
 			}
-		else if (tl->cache_hit &&
-			 tl->cache_local && !tl->thumb_path_local)
+		else
 			{
-			/* A local cache save was requested, but a valid thumb is in $HOME,
-			 * so specifically save as a local thumbnail.
-			 */
-			g_free(tl->thumb_path);
-			tl->thumb_path = NULL;
-
-			tl->cache_hit = FALSE;
-
-			if (debug) printf("thumb copied: %s\n", tl->source_path);
-
-			thumb_loader_std_save(tl, pixbuf);
+			cache_w = cache_h = THUMB_SIZE_NORMAL;
 			}
+
+		if (thumb_loader_std_scale_aspect(cache_w, cache_h, sw, sh,
+						  &thumb_w, &thumb_h))
+			{
+			pixbuf_thumb = gdk_pixbuf_scale_simple(pixbuf, thumb_w, thumb_h,
+							       (GdkInterpType)thumbnail_quality);
+			}
+		else
+			{
+			pixbuf_thumb = pixbuf;
+			g_object_ref(G_OBJECT(pixbuf_thumb));
+			}
+		
+		thumb_loader_std_save(tl, pixbuf_thumb);
+		}
+	else if (tl->cache_enable && tl->cache_local &&
+		 tl->cache_hit && !tl->thumb_path_local)
+		{
+		/* A local cache save was requested, but a valid thumb is in $HOME,
+		 * so specifically save as a local thumbnail.
+		 */
+		g_free(tl->thumb_path);
+		tl->thumb_path = NULL;
+
+		tl->cache_hit = FALSE;
+
+		if (debug) printf("thumb copied: %s\n", tl->source_path);
+
+		thumb_loader_std_save(tl, pixbuf);
 		}
 
 	if (sw <= tl->requested_width && sh <= tl->requested_height)
@@ -601,7 +595,7 @@ static void thumb_loader_std_done_cb(ImageLoader *il, gpointer data)
 
 	tl->cache_hit = (tl->thumb_path != NULL);
 
-	tl->pixbuf = thumb_loader_std_finish(tl, pixbuf);
+	tl->pixbuf = thumb_loader_std_finish(tl, pixbuf, il->shrunk);
 
 	if (tl->func_done) tl->func_done(tl, tl->data);
 }
@@ -641,18 +635,19 @@ static gint thumb_loader_std_setup(ThumbLoaderStd *tl, const gchar *path)
 {
 	tl->il = image_loader_new(path);
 
-#if 0
-	/* this will speed up jpegs by up to 3x in some cases */
-	if (tl->requested_width <= THUMB_SIZE_NORMAL &&
-	    tl->requested_height <= THUMB_SIZE_NORMAL)
+	if (thumbnail_fast)
 		{
-		image_loader_set_requested_size(tl->il, THUMB_SIZE_NORMAL, THUMB_SIZE_NORMAL);
+		/* this will speed up jpegs by up to 3x in some cases */
+		if (tl->requested_width <= THUMB_SIZE_NORMAL &&
+		    tl->requested_height <= THUMB_SIZE_NORMAL)
+			{
+			image_loader_set_requested_size(tl->il, THUMB_SIZE_NORMAL, THUMB_SIZE_NORMAL);
+			}
+		else
+			{
+			image_loader_set_requested_size(tl->il, THUMB_SIZE_LARGE, THUMB_SIZE_LARGE);
+			}
 		}
-	else
-		{
-		image_loader_set_requested_size(tl->il, THUMB_SIZE_LARGE, THUMB_SIZE_LARGE);
-		}
-#endif
 
 	image_loader_set_error_func(tl->il, thumb_loader_std_error_cb, tl);
 	if (tl->func_progress)
@@ -714,14 +709,14 @@ gint thumb_loader_std_start(ThumbLoaderStd *tl, const gchar *path)
 		{
 		gint found;
 
+		if (thumb_loader_std_fail_check(tl)) return FALSE;
+
 		tl->thumb_path = thumb_loader_std_cache_path(tl, FALSE, NULL, FALSE);
 		tl->thumb_path_local = FALSE;
 
 		found = isfile(tl->thumb_path);
 		if (found && thumb_loader_std_setup(tl, tl->thumb_path)) return TRUE;
 
-		if (thumb_loader_std_fail_check(tl)) return FALSE;
-
 		return thumb_loader_std_next_source(tl, found);
 		}
 
@@ -1117,7 +1112,7 @@ void thumb_std_maint_moved(const gchar *source, const gchar *dest)
 
 	if (thumb_std_maint_move_tail)
 		{
-		thumb_std_maint_move_tail = g_list_append(thumb_std_maint_move_tail, tm);
+		g_list_append(thumb_std_maint_move_tail, tm);
 		thumb_std_maint_move_tail = thumb_std_maint_move_tail->next;
 		}
 	else
