@@ -26,6 +26,7 @@
 
 #include "ui_fileops.h"
 #include "ui_menu.h"
+#include "ui_misc.h"
 #include "ui_utildlg.h"
 #include "ui_tabcomp.h"
 
@@ -359,7 +360,7 @@ GList *history_list_get_by_key(const gchar *key)
  *-----------------------------------------------------------------------------
  */
 
-#define BOOKMARK_KEY "bookmarks"
+#define BOOKMARK_DATA_KEY "bookmarkdata"
 #define MARKER_PATH "[path]"
 #define MARKER_ICON "[icon]"
 
@@ -376,6 +377,9 @@ struct _BookMarkData
 	void (*select_func)(const gchar *path, gpointer data);
 	gpointer select_data;
 
+	gint no_defaults;
+	gint editable;
+
 	BookButtonData *active_button;
 };
 
@@ -558,11 +562,12 @@ static void bookmark_edit_ok_cb(GenericDialog *gd, gpointer data)
 		history_list_add_to_key(p->bb->parent, new, 0);
 		}
 
+	if (path && strlen(path) > 0) tab_completion_append_to_history(p->path_entry, path);
+	if (icon && strlen(icon) > 0) tab_completion_append_to_history(p->icon_entry, icon);
+
 	g_free(path);
 	g_free(new);
 
-	if (icon && strlen(icon) > 0) tab_completion_append_to_history(p->icon_entry, icon);
-
 	bookmark_populate_all(p->bb->parent);
 }
 
@@ -576,31 +581,26 @@ static void bookmark_edit(const gchar *key, const gchar *text, GtkWidget *parent
 	GtkWidget *label;
 	const gchar *icon;
 	
-	if (!key) key = BOOKMARK_KEY;
+	if (!key) key = "bookmarks";
 
 	p = g_new0(BookPropData, 1);
 
 	p->bb = bookmark_from_string(text);
 	p->bb->parent = g_strdup(key);
 
-	gd = generic_dialog_new(_("Edit Bookmark"), NULL, "SLIK", "bookmark_edit",
+	gd = generic_dialog_new(_("Edit Bookmark"), PACKAGE, "bookmark_edit",
 				parent, TRUE,
 				bookmark_edit_cancel_cb, p);
 	g_signal_connect(G_OBJECT(gd->dialog), "destroy",
 			 G_CALLBACK(bookmark_edit_destroy_cb), p);
 
-	generic_dialog_add_stock(gd, NULL, GTK_STOCK_OK,
-				 bookmark_edit_ok_cb, TRUE);
+	generic_dialog_add_message(gd, NULL, _("Edit Bookmark"), NULL);
 
-	table = gtk_table_new(3, 2, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
-	gtk_box_pack_start(GTK_BOX(gd->vbox), table, TRUE, TRUE, 0);
-	gtk_widget_show(table);
+	generic_dialog_add_button(gd, GTK_STOCK_OK, NULL,
+				  bookmark_edit_ok_cb, TRUE);
 
-	label = gtk_label_new(_("Name:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-		         GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show(label);
+	table = pref_table_new(gd->vbox, 3, 2, FALSE, TRUE);
+	pref_table_label(table, 0, 0, _("Name:"), 1.0);
 
 	p->name_entry = gtk_entry_new();
 	gtk_widget_set_size_request(p->name_entry, 300, -1);
@@ -609,21 +609,16 @@ static void bookmark_edit(const gchar *key, const gchar *text, GtkWidget *parent
 	generic_dialog_attach_default(gd, p->name_entry);
 	gtk_widget_show(p->name_entry);
 
-	label = gtk_label_new(_("Path:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-			 GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show(label);
+	pref_table_label(table, 0, 1, _("Path:"), 1.0);
 
-	label = tab_completion_new(&p->path_entry, p->bb->path, NULL, NULL);
+	label = tab_completion_new_with_history(&p->path_entry, p->bb->path,
+						"bookmark_path", -1, NULL, NULL);
 	tab_completion_add_select_button(p->path_entry, NULL, TRUE);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
 	generic_dialog_attach_default(gd, p->path_entry);
 	gtk_widget_show(label);
 
-	label = gtk_label_new(_("Icon:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-			 GTK_FILL, GTK_FILL, 0, 0);
-	gtk_widget_show(label);
+	pref_table_label(table, 0, 2, _("Icon:"), 1.0);
 
 	icon = p->bb->icon;
 	if (!icon) icon = "";
@@ -644,6 +639,8 @@ static void bookmark_move(BookMarkData *bm, GtkWidget *button, gint direction)
 	GList *list;
 	gchar *key_holder;
 
+	if (!bm->editable) return;
+
 	b = g_object_get_data(G_OBJECT(button), "bookbuttondata");
 	if (!b) return;
 
@@ -718,13 +715,13 @@ static void bookmark_menu_popup(BookMarkData *bm, GtkWidget *button,
 	bm->active_button = b;
 
 	menu = popup_menu_short_lived();
-	menu_item_add_stock(menu, _("_Properties..."), GTK_STOCK_PROPERTIES,
+	menu_item_add_stock_sensitive(menu, _("_Properties..."), GTK_STOCK_PROPERTIES, bm->editable,
 		      G_CALLBACK(bookmark_menu_prop_cb), bm);
-	menu_item_add_stock(menu, _("Move _up"), GTK_STOCK_GO_UP,
+	menu_item_add_stock_sensitive(menu, _("Move _up"), GTK_STOCK_GO_UP, bm->editable,
 		      G_CALLBACK(bookmark_menu_up_cb), bm);
-	menu_item_add_stock(menu, _("Move _down"), GTK_STOCK_GO_DOWN,
+	menu_item_add_stock_sensitive(menu, _("Move _down"), GTK_STOCK_GO_DOWN, bm->editable,
 		      G_CALLBACK(bookmark_menu_down_cb), bm);
-	menu_item_add_stock(menu, _("_Remove"), GTK_STOCK_REMOVE,
+	menu_item_add_stock_sensitive(menu, _("_Remove"), GTK_STOCK_REMOVE, bm->editable,
 		      G_CALLBACK(bookmark_menu_remove_cb), bm);
 
 	if (local)
@@ -851,7 +848,7 @@ static void bookmark_populate(BookMarkData *bm)
 		gtk_widget_destroy(widget);
 		}
 
-	if (!history_list_get_by_key(bm->key))
+	if (!bm->no_defaults && !history_list_get_by_key(bm->key))
 		{
 		gchar *buf;
 		gchar *path;
@@ -897,7 +894,7 @@ static void bookmark_populate(BookMarkData *bm)
 		b = bookmark_from_string(work->data);
 		if (b)
 			{
-			GtkWidget *vbox;
+			GtkWidget *box;
 
 			b->button = gtk_button_new();
 			gtk_button_set_relief(GTK_BUTTON(b->button), GTK_RELIEF_NONE);
@@ -907,9 +904,9 @@ static void bookmark_populate(BookMarkData *bm)
 			g_object_set_data_full(G_OBJECT(b->button), "bookbuttondata",
 					       b, (GDestroyNotify)bookmark_free);
 
-			vbox = gtk_vbox_new(FALSE, 0);
-			gtk_container_add(GTK_CONTAINER(b->button), vbox);
-			gtk_widget_show(vbox);
+			box = gtk_hbox_new(FALSE, PREF_PAD_BUTTON_GAP);
+			gtk_container_add(GTK_CONTAINER(b->button), box);
+			gtk_widget_show(box);
 
 			if (b->icon)
 				{
@@ -943,11 +940,11 @@ static void bookmark_populate(BookMarkData *bm)
 				{
 				b->image = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
 				}
-			gtk_container_add(GTK_CONTAINER(vbox), b->image);
+			gtk_box_pack_start(GTK_BOX(box), b->image, FALSE, FALSE, 0);
 			gtk_widget_show(b->image);
 
 			b->label = gtk_label_new(b->name);
-			gtk_container_add(GTK_CONTAINER(vbox), b->label);
+			gtk_box_pack_start(GTK_BOX(box), b->label, FALSE, FALSE, 0);
 			gtk_widget_show(b->label);
 
 			g_signal_connect(G_OBJECT(b->button), "clicked",
@@ -1000,6 +997,8 @@ static void bookmark_dnd_get_data(GtkWidget *widget,
 	GList *list = NULL;
 	GList *work;
 
+	if (!bm->editable) return;
+
 	switch (info)
 		{
 		case TARGET_URI_LIST:
@@ -1042,7 +1041,7 @@ GtkWidget *bookmark_list_new(const gchar *key,
 	GtkWidget *scrolled;
 	BookMarkData *bm;
 	
-	if (!key) key = BOOKMARK_KEY;
+	if (!key) key = "bookmarks";
 
 	bm = g_new0(BookMarkData, 1);
 	bm->key = g_strdup(key);
@@ -1050,6 +1049,9 @@ GtkWidget *bookmark_list_new(const gchar *key,
 	bm->select_func = select_func;
 	bm->select_data = select_data;
 
+	bm->no_defaults = FALSE;
+	bm->editable = TRUE;
+
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
@@ -1062,8 +1064,8 @@ GtkWidget *bookmark_list_new(const gchar *key,
 
 	g_signal_connect(G_OBJECT(bm->box), "destroy",
 			 G_CALLBACK(bookmark_list_destroy), bm);
-	g_object_set_data(G_OBJECT(bm->box), "bookmarkdata", bm);
-	g_object_set_data(G_OBJECT(scrolled), "bookmarkdata", bm);
+	g_object_set_data(G_OBJECT(bm->box), BOOKMARK_DATA_KEY, bm);
+	g_object_set_data(G_OBJECT(scrolled), BOOKMARK_DATA_KEY, bm);
 	bm->widget = scrolled;
 
 	gtk_drag_dest_set(scrolled,
@@ -1078,12 +1080,49 @@ GtkWidget *bookmark_list_new(const gchar *key,
 	return scrolled;
 }
 
+void bookmark_list_set_key(GtkWidget *list, const gchar *key)
+{
+	BookMarkData *bm;
+
+	if (!list || !key) return;
+
+	bm = g_object_get_data(G_OBJECT(list), BOOKMARK_DATA_KEY);
+	if (!bm) return;
+
+	if (bm->key && strcmp(bm->key, key) == 0) return;
+
+	g_free(bm->key);
+	bm->key = g_strdup(key);
+
+	bookmark_populate(bm);
+}
+
+void bookmark_list_set_no_defaults(GtkWidget *list, gint no_defaults)
+{
+	BookMarkData *bm;
+
+	bm = g_object_get_data(G_OBJECT(list), BOOKMARK_DATA_KEY);
+	if (!bm) return;
+
+	bm->no_defaults = no_defaults;
+}
+
+void bookmark_list_set_editable(GtkWidget *list, gint editable)
+{
+	BookMarkData *bm;
+
+	bm = g_object_get_data(G_OBJECT(list), BOOKMARK_DATA_KEY);
+	if (!bm) return;
+
+	bm->editable = editable;
+}
+
 void bookmark_list_add(GtkWidget *list, const gchar *name, const gchar *path)
 {
 	BookMarkData *bm;
 	gchar *buf;
 
-	bm = g_object_get_data(G_OBJECT(list), "bookmarkdata");
+	bm = g_object_get_data(G_OBJECT(list), BOOKMARK_DATA_KEY);
 	if (!bm) return;
 
 	buf = bookmark_string(name, path, NULL);
@@ -1128,30 +1167,42 @@ GtkWidget *history_combo_new(GtkWidget **entry, const gchar *text,
 			     const gchar *history_key, gint max_levels)
 {
 	HistoryComboData *hc;
-	GList *list;
+	GList *work;
+	gint n = 0;
 
 	hc = g_new0(HistoryComboData, 1);
 	hc->history_key = g_strdup(history_key);
 	hc->history_levels = max_levels;
 
-	hc->combo = gtk_combo_new();
+	hc->combo = gtk_combo_box_entry_new_text();
+#if 0
 	gtk_combo_set_case_sensitive(GTK_COMBO(hc->combo), TRUE);
 	gtk_combo_set_use_arrows(GTK_COMBO(hc->combo), FALSE);
+#endif
 
-	hc->entry = GTK_COMBO(hc->combo)->entry;
+	hc->entry = GTK_BIN(hc->combo)->child;
 
 	g_object_set_data(G_OBJECT(hc->combo), "history_combo_data", hc);
 	g_object_set_data(G_OBJECT(hc->entry), "history_combo_data", hc);
 	g_signal_connect(G_OBJECT(hc->combo), "destroy",
 			 G_CALLBACK(history_combo_destroy), hc);
 
-	list = history_list_get_by_key(hc->history_key);
-	if (list)
+	work = history_list_get_by_key(hc->history_key);
+	while (work)
 		{
-		gtk_combo_set_popdown_strings(GTK_COMBO(hc->combo), list);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(hc->combo), (gchar *)work->data);
+		work = work->next;
+		n++;
 		}
 
-	if (text) gtk_entry_set_text(GTK_ENTRY(hc->entry), text);
+	if (text)
+		{
+		gtk_entry_set_text(GTK_ENTRY(hc->entry), text);
+		}
+	else if (n > 0)
+		{
+		gtk_combo_box_set_active(GTK_COMBO_BOX(hc->combo), 0);
+		}
 
 	if (entry) *entry = hc->entry;
 	return hc->combo;
@@ -1183,15 +1234,21 @@ void history_combo_append_history(GtkWidget *widget, const gchar *text)
 
 	if (new_text && strlen(new_text) > 0)
 		{
-		GList *list;
+		GtkTreeModel *store;
+		GList *work;
 
 		history_list_add_to_key(hc->history_key, new_text, hc->history_levels);
-		list = history_list_get_by_key(hc->history_key);
-		if (list) gtk_combo_set_popdown_strings(GTK_COMBO(hc->combo), list);
 
-		if (strcmp(new_text, gtk_entry_get_text(GTK_ENTRY(hc->entry))) != 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(hc->combo), -1);
+
+		store = gtk_combo_box_get_model(GTK_COMBO_BOX(hc->combo));
+		gtk_list_store_clear(GTK_LIST_STORE(store));
+
+		work = history_list_get_by_key(hc->history_key);
+		while (work)
 			{
-			gtk_entry_set_text(GTK_ENTRY(hc->entry), new_text);
+			gtk_combo_box_append_text(GTK_COMBO_BOX(hc->combo), (gchar *)work->data);
+			work = work->next;
 			}
 		}
 
