@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2006 John Ellis
+ * (C) 2004 John Ellis
  *
  * Author: John Ellis
  *
@@ -300,26 +300,6 @@ GenericDialog *file_util_warning_dialog(const gchar *heading, const gchar *messa
 	return gd;
 }
 
-static gint filename_base_length(const gchar *name)
-{
-	gint n;
-
-	if (!name) return 0;
-
-	n = strlen(name);
-
-	if (filter_name_exists(name))
-		{
-		const gchar *ext;
-
-		ext = extension_from_path(name);
-		if (ext) n -= strlen(ext);
-		}
-
-	return n;
-}
-
-
 /*
  *--------------------------------------------------------------------------
  * Move and Copy routines
@@ -479,15 +459,7 @@ static void file_util_move_multiple_rename_cb(GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive(fdm->rename_entry, fdm->rename);
 	gtk_widget_set_sensitive(fdm->yes_all_button, !fdm->rename);
 
-	if (fdm->rename)
-		{
-		const gchar *name;
-
-		gtk_widget_grab_focus(fdm->rename_entry);
-
-		name = gtk_entry_get_text(GTK_ENTRY(fdm->rename_entry));
-		gtk_editable_select_region(GTK_EDITABLE(fdm->rename_entry), 0, filename_base_length(name));
-		}
+	if (fdm->rename) gtk_widget_grab_focus(fdm->rename_entry);
 }
 
 static GenericDialog *file_util_move_multiple_confirm_dialog(FileDataMult *fdm)
@@ -792,15 +764,7 @@ static void file_util_move_single_rename_cb(GtkWidget *widget, gpointer data)
 	fds->rename = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 	gtk_widget_set_sensitive(fds->rename_entry, fds->rename);
 
-	if (fds->rename)
-		{
-		const gchar *name;
-
-		gtk_widget_grab_focus(fds->rename_entry);
-
-		name = gtk_entry_get_text(GTK_ENTRY(fds->rename_entry));
-		gtk_editable_select_region(GTK_EDITABLE(fds->rename_entry), 0, filename_base_length(name));
-		}
+	if (fds->rename) gtk_widget_grab_focus(fds->rename_entry);
 }
 
 static void file_util_move_single(FileDataSingle *fds)
@@ -928,11 +892,6 @@ static void file_util_move_do(FileDialog *fd)
 
 static void file_util_move_check(FileDialog *fd)
 {
-	if (fd->dest_path && strcmp(fd->dest_path, "~") == 0)
-		{
-		gtk_entry_set_text(GTK_ENTRY(fd->entry), homedir());
-		}
-
 	if (fd->multiple_files && !isdir(fd->dest_path))
 		{
 		if (isfile(fd->dest_path))
@@ -948,24 +907,6 @@ static void file_util_move_check(FileDialog *fd)
 		return;
 		}
 
-	if (!fd->dest_path || fd->dest_path[0] != '/')
-		{
-		if (fd->source_path)
-			{
-			gchar *base;
-			gchar *path;
-
-			base = remove_level_from_path(fd->source_path);
-			path = concat_dir_and_file(base, fd->dest_path);
-
-			gtk_entry_set_text(GTK_ENTRY(fd->entry), path);
-
-			g_free(path);
-			g_free(base);
-			}
-		return;
-		}
-
 	file_util_move_do(fd);
 }
 
@@ -1934,7 +1875,6 @@ static gboolean file_util_rename_multiple_select_cb(GtkTreeSelection *selection,
 	if (GTK_WIDGET_VISIBLE(rd->rename_box))
 		{
 		gtk_widget_grab_focus(rd->rename_entry);
-		gtk_editable_select_region(GTK_EDITABLE(rd->rename_entry), 0, filename_base_length(name));
 		}
 
 	return TRUE;
@@ -2085,7 +2025,6 @@ static void file_util_rename_multiple_do(GList *source_list, GtkWidget *parent)
 	GtkWidget *table;
 	GtkWidget *combo;
 	GList *work;
-	const gchar *name;
 
 	rd = g_new0(RenameDataMult, 1);
 
@@ -2174,13 +2113,10 @@ static void file_util_rename_multiple_do(GList *source_list, GtkWidget *parent)
 	pref_table_label(table, 0, 1, _("New name:"), 1.0);
 
 	rd->rename_entry = gtk_entry_new();
-	gtk_table_attach(GTK_TABLE(table), rd->rename_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_entry_set_text(GTK_ENTRY(rd->rename_entry), filename_from_path(rd->fd->source_path));
+	gtk_table_attach(GTK_TABLE(table), rd->rename_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, FALSE, 0, 0);
 	generic_dialog_attach_default(GENERIC_DIALOG(rd->fd), rd->rename_entry);
 	gtk_widget_grab_focus(rd->rename_entry);
-
-	name = filename_from_path(rd->fd->source_path);
-	gtk_entry_set_text(GTK_ENTRY(rd->rename_entry), name);
-	gtk_editable_select_region(GTK_EDITABLE(rd->rename_entry), 0, filename_base_length(name));
 	gtk_widget_show(rd->rename_entry);
 
 	rd->auto_box = gtk_vbox_new(FALSE, PREF_PAD_GAP);
@@ -2200,7 +2136,7 @@ static void file_util_rename_multiple_do(GList *source_list, GtkWidget *parent)
 	box2 = furm_simple_vlabel(hbox, _("Start #"), FALSE);
 
 	rd->auto_spin_start = pref_spin_new(box2, NULL, NULL,
-					    0.0, 1000000.0, 1.0, 0, 1.0,
+					    0.0, 1000000.0, 1.0, 0, 0.0,
 					    G_CALLBACK(file_util_rename_multiple_preview_adj_cb), rd);
 
 	box2 = furm_simple_vlabel(hbox, _("End text"), TRUE);
@@ -2309,7 +2245,6 @@ static void file_util_rename_single_do(const gchar *source_path, GtkWidget *pare
 {
 	FileDialog *fd;
 	GtkWidget *table;
-	const gchar *name;
 
 	fd = file_util_file_dlg(_("Rename - GQview"), "GQview", "dlg_rename", parent,
 			     file_util_rename_single_close_cb, NULL);
@@ -2330,13 +2265,11 @@ static void file_util_rename_single_do(const gchar *source_path, GtkWidget *pare
 	pref_table_label(table, 0, 1, _("New name:"), 1.0);
 
 	fd->entry = gtk_entry_new();
-	gtk_table_attach(GTK_TABLE(table), fd->entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), fd->entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, FALSE, 0, 0);
+	gtk_entry_set_text(GTK_ENTRY(fd->entry), filename_from_path(fd->source_path));
+	gtk_editable_select_region(GTK_EDITABLE(fd->entry), 0, strlen(gtk_entry_get_text(GTK_ENTRY(fd->entry))));
 	generic_dialog_attach_default(GENERIC_DIALOG(fd), fd->entry);
 	gtk_widget_grab_focus(fd->entry);
-
-	name = filename_from_path(fd->source_path);
-	gtk_entry_set_text(GTK_ENTRY(fd->entry), name);
-	gtk_editable_select_region(GTK_EDITABLE(fd->entry), 0, filename_base_length(name));
 	gtk_widget_show(fd->entry);
 
 	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
@@ -2453,43 +2386,3 @@ void file_util_create_dir(const gchar *path, GtkWidget *parent)
 	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
 }
 
-gint file_util_rename_dir(const gchar *old_path, const gchar *new_path, GtkWidget *parent)
-{
-	const gchar *old_name;
-	const gchar *new_name;
-
-	if (!old_path || !new_path || !isdir(old_path)) return FALSE;
-
-	old_name = filename_from_path(old_path);
-	new_name = filename_from_path(new_path);
-
-	if (isdir(new_path))
-		{
-		gchar *text = g_strdup_printf(_("The folder:\n%s\nalready exists."), new_name);
-		file_util_warning_dialog(_("Folder exists"), text, GTK_STOCK_DIALOG_INFO, parent);
-		g_free(text);
-
-		return FALSE;
-		}
-
-	if (isname(new_path))
-		{
-		gchar *text = g_strdup_printf(_("The path:\n%s\nalready exists as a file."), new_name);
-		file_util_warning_dialog(_("Rename failed"), text, GTK_STOCK_DIALOG_INFO,parent);
-		g_free(text);
-
-		return FALSE;
-		}
-
-	if (!rename_file(old_path, new_path))
-		{
-		gchar *text = g_strdup_printf(_("Failed to rename %s to %s."), old_name, new_name);
-		file_util_warning_dialog(_("Rename failed"), text, GTK_STOCK_DIALOG_ERROR, parent);
-		g_free(text);
-
-		return FALSE;
-		}
-
-	return TRUE;
-}
-
